"use strict";
/* license_header */
Object.defineProperty(exports, "__esModule", { value: true });
var isError = require("lodash.iserror");
var findLastIndex = require("lodash.findlastindex");
var last = require("lodash.last");
var isString = require("lodash.isstring");
var isFunction = require("lodash.isfunction");
var path = require("path");
/**
 * Construit un logger avec la catégory demandée
 * @param category
 * @param getLoggerFn La fonction permettant de charger le logger, cette fonction est différente
 *            selon le client ou le serveur, c'est pour cette raison quelle doit être injectée
 */
var Logger = /** @class */ (function () {
    function Logger(category) {
        /*if (!(this.log4jsLogger instanceof Logger)) {
            return new Logger(category);
        }*/
        this.category = category;
    }
    Logger.getLogger = function (category) {
        return new Logger(category);
    };
    /**
     * Récupère le logger. Si côté client alors utilise log4js, si côté serveur alors utilise
     * log4js-node
     */
    Logger.prototype.buildLogger = function (category) {
        throw new Error("The log building function should be injected before use log : " +
            "Logger.prototype.buildLogger = (ServerLog|ClientLog).getLoggerBuilder(logConfig)");
    };
    Logger.prototype.fatal = function (message) {
        this.logInternal("fatal", arguments);
    };
    Logger.prototype.error = function (message) {
        this.logInternal("error", arguments);
    };
    Logger.prototype.warn = function (message) {
        this.logInternal("warn", arguments);
    };
    Logger.prototype.info = function (message) {
        this.logInternal("info", arguments);
    };
    Logger.prototype.debug = function () {
        this.logInternal("debug", arguments);
    };
    Logger.prototype.trace = function (message) {
        this.logInternal("trace", arguments);
    };
    Logger.prototype.deprecated = function (message) {
        this.logInternal("deprecated", arguments);
    };
    /**
     * Récupère le nom de la fonction appelante,
     * [mantis 0055464] en évitant de ramener l'appel du logger, qui ne nous intéresse pas :
     * on remonte la pile d'appels en cherchant le code applicatif à l'origine de la log.
     *
     * Si la "vraie" fonction appelante n'est pas trouvée dans la pile,
     * alors par défaut on utilise le paramètre d'entrée callStackSize
     * qui indique le nombre arbitraire de niveaux à remonter dans la pile
     * pour avoir la fonction appelante
     *
     */
    Logger.getFunctionName = function (callStackSize) {
        var notTyppedError = Error;
        var orig = notTyppedError.prepareStackTrace;
        var err;
        var functionName = "";
        if (typeof notTyppedError.captureStackTrace === "function") {
            // Chrome/Node
            notTyppedError.prepareStackTrace = function (_, stack) {
                return stack;
            };
            err = new notTyppedError();
            notTyppedError.captureStackTrace(err, Logger.getFunctionName);
            functionName = "unknownFunction";
            if (err.stack) {
                // Remonter la stack jusqu'a la fonction appellante du logger
                // D'abord, on cherche le premier appel au logger dans la stack (en partant du haut)
                var lastLoggerStackIndex = findLastIndex(err.stack, function (o) {
                    return o.getTypeName && o.getTypeName() === "Logger";
                });
                // si on a trouvé l'appel au logger dans la stack :
                if (lastLoggerStackIndex > 0 && err.stack.length > lastLoggerStackIndex + 1) {
                    // on remonte d'un cran pour avoir le nom de la fonction appelante
                    var hornetCall = err.stack[lastLoggerStackIndex + 1];
                    functionName = hornetCall.getFunctionName();
                    // parfois, le nom de la fonction est vide (cas des fonctions déclarées dynamiquement)
                    if (!functionName) {
                        // dans ce cas on affiche "anonymous" avec le nom du fichier et le numéro de ligne+colonne
                        var filename = hornetCall.getFileName() || "no source file";
                        functionName = "anonymous:".concat(last(filename.split(path.sep)));
                        // functionName = hornetCall.toString();
                    }
                    functionName = functionName.concat(":").concat(hornetCall.getLineNumber())
                        .concat(":").concat(hornetCall.getColumnNumber());
                }
                else if (err.stack.length >= callStackSize) {
                    // traitement par défaut : on va chercher dans la pile avec l'index fourni en paramètre (callStackSize)
                    functionName = err.stack[callStackSize - 1].getFunctionName();
                }
            }
            notTyppedError.prepareStackTrace = orig;
        }
        else {
            // Firefox
            var e = new notTyppedError().stack;
            if (e) {
                var callstack = e.split("\n");
                if (callstack.length > callStackSize) {
                    functionName = callstack[callStackSize];
                }
            }
        }
        return functionName;
    };
    /**
     * Appelle la fonction de journalisation du logger en ajoutant le nom de la fonction appelant et si
     * disponible l'id de traitement
     *
     * @param niveau de log
     * @param [, arg1[, arg2[, ...]]] Une liste d'arguments à logguer
     */
    Logger.prototype.log = function (level) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (!isString(level)) {
            args.unshift(level);
            level = "error";
        }
        this.logInternal.call(this, level, args);
    };
    /**
     * Appelle la fonction de journalisation du logger en ajoutant le nom de la fonction appelant et si
     * disponible l'id de traitement
     *
     * @param niveau de log
     * @param logArguments: Un tableau des objets (string, error ou object) à logguer
     */
    Logger.prototype.logInternal = function (level, logArguments) {
        if (!this.log4jsLogger) {
            this.buildLogger(this.category);
        }
        var logFn;
        var stack;
        var stackLogEnabled;
        stackLogEnabled = this.log4jsLogger.isStackErrorLogEnabled && this.log4jsLogger.isStackErrorLogEnabled() === "true";
        switch (level.toLowerCase()) {
            case "trace":
                if (this.log4jsLogger.isTraceEnabled()) {
                    logFn = this.log4jsLogger.trace;
                    stack = stackLogEnabled ? this.searchStack(logArguments) : null;
                }
                break;
            case "debug":
                if (this.log4jsLogger.isDebugEnabled()) {
                    logFn = this.log4jsLogger.debug;
                    stack = stackLogEnabled ? this.searchStack(logArguments) : null;
                }
                break;
            case "info":
                if (this.log4jsLogger.isInfoEnabled()) {
                    logFn = this.log4jsLogger.info;
                    stack = stackLogEnabled ? this.searchStack(logArguments) : null;
                }
                break;
            case "warn":
                if (this.log4jsLogger.isWarnEnabled()) {
                    logFn = this.log4jsLogger.warn;
                    stack = stackLogEnabled ? this.searchStack(logArguments) : null;
                }
                break;
            case "fatal":
                if (this.log4jsLogger.isFatalEnabled()) {
                    logFn = this.log4jsLogger.fatal;
                    stack = this.searchStack(logArguments);
                }
                break;
            case "deprecated":
                if (this.log4jsLogger.isWarnEnabled() && (process.env.NODE_ENV !== "production")) {
                    logFn = this.log4jsLogger.warn;
                    stack = this.searchStack(logArguments);
                }
                break;
            default:
                if (this.log4jsLogger.isErrorEnabled()) {
                    logFn = this.log4jsLogger.error;
                    stack = this.searchStack(logArguments);
                }
        }
        if (logFn) {
            if (stack && this.log4jsLogger.setStack) {
                this.log4jsLogger.setStack(stack);
            }
            // On a bien besoin de logguer
            var parameters = Array.prototype.slice.call(logArguments);
            var message = parameters.map(this.mappingObjectToString.bind(this)).join(" ");
            // Et enfin on log réellement
            logFn.call(this.log4jsLogger, message);
        }
    };
    /**
     * Méthode de recherche de stack depuis les arguments passés dans le logger
     * @param args
     * @returns {{}}
     */
    Logger.prototype.searchStack = function (args) {
        var stack = {};
        if (args) {
            Object.keys(args).map(function (key) {
                stack = args[key] && args[key].error && args[key].error.stack || args[key] && args[key].stack || null;
            });
        }
        return stack;
    };
    Logger.prototype.mappingObjectToString = function (arg) {
        if (isString(arg)) {
            return arg;
        }
        if (isError(arg) && arg["toLog"]) {
            return arg.toLog();
        }
        else {
            try {
                // return JSON.stringify(arg);
                return JSON.stringify(arg, function (key, value) {
                    if (isFunction(value)) {
                        return "[Function" + (value.name ? ": " + value.name : "") + "]";
                    }
                    else if (value instanceof RegExp) {
                        return "_PxEgEr_" + value;
                    }
                    return value;
                });
            }
            catch (err) {
                return "<stringifyErr>";
            }
        }
    };
    Logger.log4js = typeof window !== "undefined" ? require("./extend/log4js") : undefined;
    return Logger;
}());
exports.Logger = Logger;

//# sourceMappingURL=logger.js.map
